@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Designer of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qdesigner_menu_p.h"
 #include "qdesigner_menubar_p.h"
@@ -56,11 +31,9 @@
 #include <QtCore/qtimer.h>
 #include <QtCore/qdebug.h>
 
-Q_DECLARE_METATYPE(QAction*)
-
 QT_BEGIN_NAMESPACE
 
-using namespace qdesigner_internal;
+using namespace Qt::StringLiterals;
 
 // give the user a little more space to click on the sub menu rectangle
 static inline void extendClickableArea(QRect *subMenuRect, Qt::LayoutDirection dir)
@@ -78,10 +51,10 @@ static inline void extendClickableArea(QRect *subMenuRect, Qt::LayoutDirection d
 
 QDesignerMenu::QDesignerMenu(QWidget *parent) :
     QMenu(parent),
-    m_subMenuPixmap(QPixmap(QStringLiteral(":/qt-project.org/formeditor/images/submenu.png"))),
+    m_subMenuPixmap(QPixmap(u":/qt-project.org/formeditor/images/submenu.png"_s)),
     m_currentIndex(0),
-    m_addItem(new SpecialMenuAction(this)),
-    m_addSeparator(new SpecialMenuAction(this)),
+    m_addItem(new qdesigner_internal::SpecialMenuAction(this)),
+    m_addSeparator(new qdesigner_internal::SpecialMenuAction(this)),
     m_showSubMenuTimer(new QTimer(this)),
     m_deactivateWindowTimer(new QTimer(this)),
     m_adjustSizeTimer(new QTimer(this)),
@@ -104,7 +77,7 @@ QDesignerMenu::QDesignerMenu(QWidget *parent) :
 
     connect(m_deactivateWindowTimer, &QTimer::timeout, this, &QDesignerMenu::slotDeactivateNow);
 
-    m_editor->setObjectName(QStringLiteral("__qt__passive_editor"));
+    m_editor->setObjectName(u"__qt__passive_editor"_s);
     m_editor->hide();
 
     m_editor->installEventFilter(this);
@@ -152,6 +125,8 @@ bool QDesignerMenu::handleEvent(QWidget *widget, QEvent *event)
 
 void QDesignerMenu::startDrag(const QPoint &pos, Qt::KeyboardModifiers modifiers)
 {
+    using namespace qdesigner_internal;
+
     const int index = findAction(pos);
     if (index >= realActionCount())
         return;
@@ -161,7 +136,7 @@ void QDesignerMenu::startDrag(const QPoint &pos, Qt::KeyboardModifiers modifiers
     QDesignerFormWindowInterface *fw = formWindow();
     const Qt::DropAction dropAction = (modifiers & Qt::ControlModifier) ? Qt::CopyAction : Qt::MoveAction;
     if (dropAction == Qt::MoveAction) {
-        RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
+        auto *cmd = new RemoveActionFromCommand(fw);
         cmd->init(this, action, actions().at(index + 1));
         fw->commandHistory()->push(cmd);
     }
@@ -176,7 +151,7 @@ void QDesignerMenu::startDrag(const QPoint &pos, Qt::KeyboardModifiers modifiers
     if (drag->exec(dropAction) == Qt::IgnoreAction) {
         if (dropAction == Qt::MoveAction) {
             QAction *previous = safeActionAt(index);
-            InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+            auto *cmd = new InsertActionIntoCommand(fw);
             cmd->init(this, action, previous);
             fw->commandHistory()->push(cmd);
         }
@@ -460,7 +435,7 @@ bool QDesignerMenu::handleContextMenuEvent(QWidget *, QContextMenuEvent *event)
 
     const int index = findAction(mapFromGlobal(event->globalPos()));
     QAction *action = safeActionAt(index);
-    if (qobject_cast<SpecialMenuAction*>(action))
+    if (qobject_cast<qdesigner_internal::SpecialMenuAction*>(action))
         return true;
 
     QMenu menu;
@@ -502,14 +477,14 @@ void QDesignerMenu::slotAddSeparator()
     fw->beginCommand(tr("Add separator"));
     QAction *sep = createAction(QString(), true);
 
-    InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+    auto *cmd = new qdesigner_internal::InsertActionIntoCommand(fw);
     cmd->init(this, sep, action_before);
     fw->commandHistory()->push(cmd);
 
     if (parentMenu()) {
         QAction *parent_action = parentMenu()->currentAction();
         if (parent_action->menu() == nullptr) {
-            CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+            auto *cmd = new qdesigner_internal::CreateSubmenuCommand(fw);
             cmd->init(parentMenu(), parentMenu()->currentAction());
             fw->commandHistory()->push(cmd);
         }
@@ -533,7 +508,7 @@ void QDesignerMenu::deleteAction(QAction *a)
         action_before = safeActionAt(pos + 1);
 
     QDesignerFormWindowInterface *fw = formWindow();
-    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
+    auto *cmd = new qdesigner_internal::RemoveActionFromCommand(fw);
     cmd->init(this, a, action_before);
     fw->commandHistory()->push(cmd);
 }
@@ -549,7 +524,7 @@ QRect QDesignerMenu::subMenuPixmapRect(QAction *action) const
 bool QDesignerMenu::hasSubMenuPixmap(QAction *action) const
 {
     return action != nullptr
-            && qobject_cast<SpecialMenuAction*>(action) == 0
+            && qobject_cast<qdesigner_internal::SpecialMenuAction*>(action) == nullptr
             && !action->isSeparator()
             && !action->menu()
             && canCreateSubMenu(action);
@@ -563,6 +538,8 @@ void QDesignerMenu::showEvent ( QShowEvent * event )
 
 void QDesignerMenu::paintEvent(QPaintEvent *event)
 {
+    using namespace qdesigner_internal;
+
     QMenu::paintEvent(event);
 
     QPainter p(this);
@@ -697,7 +674,7 @@ QDesignerMenu::ActionDragCheck QDesignerMenu::checkAction(QAction *action) const
     if (!action || (action->menu() && action->menu()->parentWidget() != const_cast<QDesignerMenu*>(this)))
         return NoActionDrag; // menu action!! nothing to do
 
-    if (!Utils::isObjectAncestorOf(formWindow()->mainContainer(), action))
+    if (!qdesigner_internal::Utils::isObjectAncestorOf(formWindow()->mainContainer(), action))
         return NoActionDrag; // the action belongs to another form window
 
     if (actions().contains(action))
@@ -708,7 +685,7 @@ QDesignerMenu::ActionDragCheck QDesignerMenu::checkAction(QAction *action) const
 
 void QDesignerMenu::dragEnterEvent(QDragEnterEvent *event)
 {
-    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    auto *d = qobject_cast<const qdesigner_internal::ActionRepositoryMimeData*>(event->mimeData());
     if (!d || d->actionList().isEmpty()) {
         event->ignore();
         return;
@@ -740,7 +717,7 @@ void QDesignerMenu::dragMoveEvent(QDragMoveEvent *event)
         return;
     }
 
-    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    auto *d = qobject_cast<const qdesigner_internal::ActionRepositoryMimeData*>(event->mimeData());
     if (!d || d->actionList().isEmpty()) {
         event->ignore();
         return;
@@ -785,7 +762,7 @@ void QDesignerMenu::dropEvent(QDropEvent *event)
     m_dragging = false;
 
     QDesignerFormWindowInterface *fw = formWindow();
-    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    auto *d = qobject_cast<const qdesigner_internal::ActionRepositoryMimeData*>(event->mimeData());
     if (!d || d->actionList().isEmpty()) {
         event->ignore();
         return;
@@ -797,7 +774,7 @@ void QDesignerMenu::dropEvent(QDropEvent *event)
         index = qMin(index, actions().size() - 1);
 
         fw->beginCommand(tr("Insert action"));
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+        auto *cmd = new qdesigner_internal::InsertActionIntoCommand(fw);
         cmd->init(this, action, safeActionAt(index));
         fw->commandHistory()->push(cmd);
 
@@ -806,7 +783,7 @@ void QDesignerMenu::dropEvent(QDropEvent *event)
         if (parentMenu()) {
             QAction *parent_action = parentMenu()->currentAction();
             if (parent_action->menu() == nullptr) {
-                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+                auto *cmd = new qdesigner_internal::CreateSubmenuCommand(fw);
                 cmd->init(parentMenu(), parentMenu()->currentAction(), action);
                 fw->commandHistory()->push(cmd);
             }
@@ -877,6 +854,8 @@ bool QDesignerMenu::hideSubMenuOnCursorKey()
 // Return false to indicate the event must be propagated to the menu bar.
 bool QDesignerMenu::showSubMenuOnCursorKey()
 {
+    using namespace qdesigner_internal;
+
     const QAction *action = currentAction();
 
     if (qobject_cast<const SpecialMenuAction*>(action) || action->isSeparator()) {
@@ -953,6 +932,8 @@ int QDesignerMenu::realActionCount() const
 
 void QDesignerMenu::selectCurrentAction()
 {
+    using namespace qdesigner_internal;
+
     QAction *action = currentAction();
     if (!action || action == m_addSeparator || action == m_addItem)
         return;
@@ -982,6 +963,8 @@ void QDesignerMenu::selectCurrentAction()
 
 void QDesignerMenu::createRealMenuAction(QAction *action)
 {
+    using namespace qdesigner_internal;
+
     if (action->menu())
         return; // nothing to do
 
@@ -998,7 +981,7 @@ void QDesignerMenu::createRealMenuAction(QAction *action)
 
     core->widgetFactory()->initialize(menu);
 
-    const QString niceObjectName = ActionEditor::actionTextToName(menu->title(), QStringLiteral("menu"));
+    const QString niceObjectName = ActionEditor::actionTextToName(menu->title(), u"menu"_s);
     menu->setObjectName(niceObjectName);
 
     core->metaDataBase()->add(menu);
@@ -1094,6 +1077,8 @@ void QDesignerMenu::slotShowSubMenuNow()
 
 void QDesignerMenu::showSubMenu(QAction *action)
 {
+    using namespace qdesigner_internal;
+
     m_showSubMenuTimer->stop();
 
     if (m_editor->isVisible() || !action || qobject_cast<SpecialMenuAction*>(action)
@@ -1147,14 +1132,14 @@ void QDesignerMenu::enterEditMode()
         fw->beginCommand(tr("Add separator"));
         QAction *sep = createAction(QString(), true);
 
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+        auto *cmd = new qdesigner_internal::InsertActionIntoCommand(fw);
         cmd->init(this, sep, safeActionAt(realActionCount()));
         fw->commandHistory()->push(cmd);
 
         if (parentMenu()) {
             QAction *parent_action = parentMenu()->currentAction();
             if (parent_action->menu() == nullptr) {
-                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+                auto *cmd = new qdesigner_internal::CreateSubmenuCommand(fw);
                 cmd->init(parentMenu(), parentMenu()->currentAction());
                 fw->commandHistory()->push(cmd);
             }
@@ -1169,6 +1154,8 @@ void QDesignerMenu::enterEditMode()
 
 void QDesignerMenu::leaveEditMode(LeaveEditMode mode)
 {
+    using namespace qdesigner_internal;
+
     if (mode == Default)
         return;
 
@@ -1182,19 +1169,19 @@ void QDesignerMenu::leaveEditMode(LeaveEditMode mode)
         Q_ASSERT(fw != nullptr);
         fw->beginCommand(QApplication::translate("Command", "Insert action"));
         action = createAction(ActionEditor::actionTextToName(m_editor->text()));
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+        auto *cmd = new qdesigner_internal::InsertActionIntoCommand(fw);
         cmd->init(this, action, currentAction());
         fw->commandHistory()->push(cmd);
     }
 
-    SetPropertyCommand *cmd = new SetPropertyCommand(fw);
-    cmd->init(action, QStringLiteral("text"), m_editor->text());
+    auto *cmd = new qdesigner_internal::SetPropertyCommand(fw);
+    cmd->init(action, u"text"_s, m_editor->text());
     fw->commandHistory()->push(cmd);
 
     if (parentMenu()) {
         QAction *parent_action = parentMenu()->currentAction();
         if (parent_action->menu() == nullptr) {
-            CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+            auto *cmd = new qdesigner_internal::CreateSubmenuCommand(fw);
             cmd->init(parentMenu(), parentMenu()->currentAction(), action);
             fw->commandHistory()->push(cmd);
         }
@@ -1245,12 +1232,14 @@ QAction *QDesignerMenu::createAction(const QString &objectName, bool separator)
 {
     QDesignerFormWindowInterface *fw = formWindow();
     Q_ASSERT(fw);
-    return ToolBarEventFilter::createAction(fw, objectName, separator);
+    return qdesigner_internal::ToolBarEventFilter::createAction(fw, objectName, separator);
 }
 
 // ### share with QDesignerMenu::swap
 bool QDesignerMenu::swap(int a, int b)
 {
+    using namespace qdesigner_internal;
+
     const int left = qMin(a, b);
     int right = qMax(a, b);
 
@@ -1273,21 +1262,21 @@ bool QDesignerMenu::swap(int a, int b)
 
     QAction *action_b_before = safeActionAt(right + 1);
 
-    RemoveActionFromCommand *cmd1 = new RemoveActionFromCommand(fw);
+    auto *cmd1 = new qdesigner_internal::RemoveActionFromCommand(fw);
     cmd1->init(this, action_b, action_b_before, false);
     fw->commandHistory()->push(cmd1);
 
     QAction *action_a_before = safeActionAt(left + 1);
 
-    InsertActionIntoCommand *cmd2 = new InsertActionIntoCommand(fw);
+    auto *cmd2 = new qdesigner_internal::InsertActionIntoCommand(fw);
     cmd2->init(this, action_b, action_a_before, false);
     fw->commandHistory()->push(cmd2);
 
-    RemoveActionFromCommand *cmd3 = new RemoveActionFromCommand(fw);
+    auto *cmd3 = new qdesigner_internal::RemoveActionFromCommand(fw);
     cmd3->init(this, action_a, action_b, false);
     fw->commandHistory()->push(cmd3);
 
-    InsertActionIntoCommand *cmd4 = new InsertActionIntoCommand(fw);
+    auto *cmd4 = new qdesigner_internal::InsertActionIntoCommand(fw);
     cmd4->init(this, action_a, action_b_before, true);
     fw->commandHistory()->push(cmd4);
 
@@ -1321,7 +1310,7 @@ void QDesignerMenu::deleteAction()
         action_before = safeActionAt(pos + 1);
 
     QDesignerFormWindowInterface *fw = formWindow();
-    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
+    auto *cmd = new qdesigner_internal::RemoveActionFromCommand(fw);
     cmd->init(this, action, action_before);
     fw->commandHistory()->push(cmd);
 
