@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Linguist of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "translator.h"
 
@@ -43,10 +18,9 @@
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
-#include <QtCore/QRegExp>
+#include <QtCore/QLocale>
 #include <QtCore/QTextStream>
 
-#include <private/qlocale_p.h>
 #include <private/qtranslator_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -77,35 +51,30 @@ QList<Translator::FileFormat> &Translator::registeredFileFormats()
 
 void Translator::addIndex(int idx, const TranslatorMessage &msg) const
 {
-    if (msg.sourceText().isEmpty() && msg.id().isEmpty()) {
-        m_ctxCmtIdx[msg.context()] = idx;
-    } else {
-        m_msgIdx[TMMKey(msg)] = idx;
-        if (!msg.id().isEmpty())
-            m_idMsgIdx[msg.id()] = idx;
-    }
+
+    m_msgIdx[TMMKey(msg)] = idx;
+    if (!msg.id().isEmpty())
+        m_idMsgIdx[msg.id()] = idx;
+
 }
 
 void Translator::delIndex(int idx) const
 {
     const TranslatorMessage &msg = m_messages.at(idx);
-    if (msg.sourceText().isEmpty() && msg.id().isEmpty()) {
-        m_ctxCmtIdx.remove(msg.context());
-    } else {
-        m_msgIdx.remove(TMMKey(msg));
-        if (!msg.id().isEmpty())
-            m_idMsgIdx.remove(msg.id());
-    }
+
+    m_msgIdx.remove(TMMKey(msg));
+    if (!msg.id().isEmpty())
+        m_idMsgIdx.remove(msg.id());
+
 }
 
 void Translator::ensureIndexed() const
 {
     if (!m_indexOk) {
         m_indexOk = true;
-        m_ctxCmtIdx.clear();
         m_idMsgIdx.clear();
         m_msgIdx.clear();
-        for (int i = 0; i < m_messages.count(); i++)
+        for (int i = 0; i < m_messages.size(); i++)
             addIndex(i, m_messages.at(i));
     }
 }
@@ -124,7 +93,7 @@ void Translator::replaceSorted(const TranslatorMessage &msg)
 
 static QString elidedId(const QString &id, int len)
 {
-    return id.length() <= len ? id : id.left(len - 5) + QLatin1String("[...]");
+    return id.size() <= len ? id : id.left(len - 5) + QLatin1String("[...]");
 }
 
 static QString makeMsgId(const TranslatorMessage &msg)
@@ -180,7 +149,7 @@ void Translator::extend(const TranslatorMessage &msg, ConversionData &cd)
 void Translator::insert(int idx, const TranslatorMessage &msg)
 {
     if (m_indexOk) {
-        if (idx == m_messages.count())
+        if (idx == m_messages.size())
             addIndex(idx, msg);
         else
             m_indexOk = false;
@@ -190,7 +159,7 @@ void Translator::insert(int idx, const TranslatorMessage &msg)
 
 void Translator::append(const TranslatorMessage &msg)
 {
-    insert(m_messages.count(), msg);
+    insert(m_messages.size(), msg);
 }
 
 void Translator::appendSorted(const TranslatorMessage &msg)
@@ -212,7 +181,7 @@ void Translator::appendSorted(const TranslatorMessage &msg)
     // Working vars
     int prevLine = 0;
     int curIdx = 0;
-    foreach (const TranslatorMessage &mit, m_messages) {
+    for (const TranslatorMessage &mit : std::as_const(m_messages)) {
         bool sameFile = mit.fileName() == msg.fileName() && mit.context() == msg.context();
         int curLine;
         if (sameFile && (curLine = mit.lineNumber()) >= prevLine) {
@@ -257,7 +226,7 @@ static QString guessFormat(const QString &filename, const QString &format)
     if (format != QLatin1String("auto"))
         return format;
 
-    foreach (const Translator::FileFormat &fmt, Translator::registeredFileFormats()) {
+    for (const Translator::FileFormat &fmt : std::as_const(Translator::registeredFileFormats())) {
         if (filename.endsWith(QLatin1Char('.') + fmt.extension, Qt::CaseInsensitive))
             return fmt.extension;
     }
@@ -267,6 +236,18 @@ static QString guessFormat(const QString &filename, const QString &format)
     return QLatin1String("ts");
 }
 
+static QString getDependencyName(const QString &filename, const QString &format)
+{
+    const QString file = QFileInfo(filename).fileName();
+    const QString fmt = guessFormat(file, format);
+
+    if (file.endsWith(QLatin1Char('.') + fmt))
+        return file.chopped(fmt.size() + 1);
+
+    // no extension in the file name
+    return file;
+}
+
 bool Translator::load(const QString &filename, ConversionData &cd, const QString &format)
 {
     cd.m_sourceDir = QFileInfo(filename).absoluteDir();
@@ -294,7 +275,7 @@ bool Translator::load(const QString &filename, ConversionData &cd, const QString
 
     QString fmt = guessFormat(filename, format);
 
-    foreach (const FileFormat &format, registeredFileFormats()) {
+    for (const FileFormat &format : std::as_const(registeredFileFormats())) {
         if (fmt == format.extension) {
             if (format.loader)
                 return (*format.loader)(*this, file, cd);
@@ -335,7 +316,7 @@ bool Translator::save(const QString &filename, ConversionData &cd, const QString
     QString fmt = guessFormat(filename, format);
     cd.m_targetDir = QFileInfo(filename).absoluteDir();
 
-    foreach (const FileFormat &format, registeredFileFormats()) {
+    for (const FileFormat &format : std::as_const(registeredFileFormats())) {
         if (fmt == format.extension) {
             if (format.saver)
                 return (*format.saver)(*this, file, cd);
@@ -349,21 +330,38 @@ bool Translator::save(const QString &filename, ConversionData &cd, const QString
     return false;
 }
 
-QString Translator::makeLanguageCode(QLocale::Language language, QLocale::Country country)
+QString Translator::makeLanguageCode(QLocale::Language language, QLocale::Territory territory)
 {
-    QString result = QLocalePrivate::languageToCode(language);
-    if (language != QLocale::C && country != QLocale::AnyCountry) {
+    QString result = QLocale::languageToCode(language);
+    if (language != QLocale::C && territory != QLocale::AnyTerritory) {
         result.append(QLatin1Char('_'));
-        result.append(QLocalePrivate::countryToCode(country));
+        result.append(QLocale::territoryToCode(territory));
     }
     return result;
 }
 
-void Translator::languageAndCountry(const QString &languageCode,
-    QLocale::Language *lang, QLocale::Country *country)
+void Translator::languageAndTerritory(QStringView languageCode, QLocale::Language *langPtr,
+                                    QLocale::Territory *territoryPtr)
 {
-    QLocale::Script script;
-    QLocalePrivate::getLangAndCountry(languageCode, *lang, script, *country);
+    QLocale::Language language = QLocale::AnyLanguage;
+    QLocale::Territory territory = QLocale::AnyTerritory;
+    auto separator = languageCode.indexOf(u'_'); // "de_DE"
+    if (separator == -1) {
+        // compatibility with older .ts files
+        separator = languageCode.indexOf(u'-'); // "de-DE"
+    }
+    if (separator != -1) {
+        language = QLocale::codeToLanguage(languageCode.left(separator));
+        territory = QLocale::codeToTerritory(languageCode.mid(separator + 1));
+    } else {
+        language = QLocale::codeToLanguage(languageCode);
+        territory = QLocale(language).territory();
+    }
+
+    if (langPtr)
+        *langPtr = language;
+    if (territoryPtr)
+        *territoryPtr = territory;
 }
 
 int Translator::find(const TranslatorMessage &msg) const
@@ -383,12 +381,12 @@ int Translator::find(const QString &context,
     const QString &comment, const TranslatorMessage::References &refs) const
 {
     if (!refs.isEmpty()) {
-        for (TMM::ConstIterator it = m_messages.constBegin(); it != m_messages.constEnd(); ++it) {
+        for (auto it = m_messages.cbegin(), end = m_messages.cend(); it != end; ++it) {
             if (it->context() == context && it->comment() == comment) {
-                foreach (const TranslatorMessage::Reference &itref, it->allReferences()) {
-                    foreach (const TranslatorMessage::Reference &ref, refs) {
+                for (const auto &itref : it->allReferences()) {
+                    for (const auto &ref : refs) {
                         if (itref == ref)
-                            return it - m_messages.constBegin();
+                            return it - m_messages.cbegin();
                     }
                 }
             }
@@ -397,15 +395,9 @@ int Translator::find(const QString &context,
     return -1;
 }
 
-int Translator::find(const QString &context) const
-{
-    ensureIndexed();
-    return m_ctxCmtIdx.value(context, -1);
-}
-
 void Translator::stripObsoleteMessages()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); )
+    for (auto it = m_messages.begin(); it != m_messages.end(); )
         if (it->type() == TranslatorMessage::Obsolete || it->type() == TranslatorMessage::Vanished)
             it = m_messages.erase(it);
         else
@@ -415,7 +407,7 @@ void Translator::stripObsoleteMessages()
 
 void Translator::stripFinishedMessages()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); )
+    for (auto it = m_messages.begin(); it != m_messages.end(); )
         if (it->type() == TranslatorMessage::Finished)
             it = m_messages.erase(it);
         else
@@ -425,7 +417,7 @@ void Translator::stripFinishedMessages()
 
 void Translator::stripUntranslatedMessages()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); )
+    for (auto it = m_messages.begin(); it != m_messages.end(); )
         if (!it->isTranslated())
             it = m_messages.erase(it);
         else
@@ -433,20 +425,18 @@ void Translator::stripUntranslatedMessages()
     m_indexOk = false;
 }
 
-bool Translator::translationsExist()
+bool Translator::translationsExist() const
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); ) {
-        if (it->isTranslated())
+    for (const auto &message : m_messages) {
+        if (message.isTranslated())
             return true;
-        else
-            ++it;
     }
     return false;
 }
 
 void Translator::stripEmptyContexts()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end();)
+    for (auto it = m_messages.begin(); it != m_messages.end(); )
         if (it->sourceText() == QLatin1String(ContextComment))
             it = m_messages.erase(it);
         else
@@ -456,7 +446,7 @@ void Translator::stripEmptyContexts()
 
 void Translator::stripNonPluralForms()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); )
+    for (auto it = m_messages.begin(); it != m_messages.end(); )
         if (!it->isPlural())
             it = m_messages.erase(it);
         else
@@ -466,9 +456,9 @@ void Translator::stripNonPluralForms()
 
 void Translator::stripIdenticalSourceTranslations()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); ) {
+    for (auto it = m_messages.begin(); it != m_messages.end(); ) {
         // we need to have just one translation, and it be equal to the source
-        if (it->translations().count() == 1 && it->translation() == it->sourceText())
+        if (it->translations().size() == 1 && it->translation() == it->sourceText())
             it = m_messages.erase(it);
         else
             ++it;
@@ -478,21 +468,21 @@ void Translator::stripIdenticalSourceTranslations()
 
 void Translator::dropTranslations()
 {
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); ++it) {
-        if (it->type() == TranslatorMessage::Finished)
-            it->setType(TranslatorMessage::Unfinished);
-        it->setTranslation(QString());
+    for (auto &message : m_messages) {
+        if (message.type() == TranslatorMessage::Finished)
+            message.setType(TranslatorMessage::Unfinished);
+        message.setTranslation(QString());
     }
 }
 
 void Translator::dropUiLines()
 {
-    QString uiXt = QLatin1String(".ui");
-    QString juiXt = QLatin1String(".jui");
-    for (TMM::Iterator it = m_messages.begin(); it != m_messages.end(); ++it) {
+    const QString uiXt = QLatin1String(".ui");
+    const QString juiXt = QLatin1String(".jui");
+    for (auto &message : m_messages) {
         QHash<QString, int> have;
         QList<TranslatorMessage::Reference> refs;
-        foreach (const TranslatorMessage::Reference &itref, it->allReferences()) {
+        for (const auto &itref : message.allReferences()) {
             const QString &fn = itref.fileName();
             if (fn.endsWith(uiXt) || fn.endsWith(juiXt)) {
                 if (++have[fn] == 1)
@@ -501,27 +491,36 @@ void Translator::dropUiLines()
                 refs.append(itref);
             }
         }
-        it->setReferences(refs);
+        message.setReferences(refs);
     }
 }
 
-struct TranslatorMessageIdPtr {
-    explicit TranslatorMessageIdPtr(const TranslatorMessage &tm)
+class TranslatorMessagePtrBase
+{
+public:
+    explicit TranslatorMessagePtrBase(const Translator *tor, int messageIndex)
+        : tor(tor), messageIndex(messageIndex)
     {
-        ptr = &tm;
     }
 
     inline const TranslatorMessage *operator->() const
     {
-        return ptr;
+        return &tor->message(messageIndex);
     }
 
-    const TranslatorMessage *ptr;
+    const Translator *tor;
+    const int messageIndex;
 };
 
-Q_DECLARE_TYPEINFO(TranslatorMessageIdPtr, Q_MOVABLE_TYPE);
+class TranslatorMessageIdPtr : public TranslatorMessagePtrBase
+{
+public:
+    using TranslatorMessagePtrBase::TranslatorMessagePtrBase;
+};
 
-inline int qHash(TranslatorMessageIdPtr tmp)
+Q_DECLARE_TYPEINFO(TranslatorMessageIdPtr, Q_RELOCATABLE_TYPE);
+
+inline size_t qHash(TranslatorMessageIdPtr tmp)
 {
     return qHash(tmp->id());
 }
@@ -531,25 +530,17 @@ inline bool operator==(TranslatorMessageIdPtr tmp1, TranslatorMessageIdPtr tmp2)
     return tmp1->id() == tmp2->id();
 }
 
-struct TranslatorMessageContentPtr {
-    explicit TranslatorMessageContentPtr(const TranslatorMessage &tm)
-    {
-        ptr = &tm;
-    }
-
-    inline const TranslatorMessage *operator->() const
-    {
-        return ptr;
-    }
-
-    const TranslatorMessage *ptr;
+class TranslatorMessageContentPtr : public TranslatorMessagePtrBase
+{
+public:
+    using TranslatorMessagePtrBase::TranslatorMessagePtrBase;
 };
 
-Q_DECLARE_TYPEINFO(TranslatorMessageContentPtr, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(TranslatorMessageContentPtr, Q_RELOCATABLE_TYPE);
 
-inline int qHash(TranslatorMessageContentPtr tmp)
+inline size_t qHash(TranslatorMessageContentPtr tmp)
 {
-    int hash = qHash(tmp->context()) ^ qHash(tmp->sourceText());
+    size_t hash = qHash(tmp->context()) ^ qHash(tmp->sourceText());
     if (!tmp->sourceText().isEmpty())
         // Special treatment for context comments (empty source).
         hash ^= qHash(tmp->comment());
@@ -569,33 +560,31 @@ inline bool operator==(TranslatorMessageContentPtr tmp1, TranslatorMessageConten
 Translator::Duplicates Translator::resolveDuplicates()
 {
     Duplicates dups;
-    QHash<TranslatorMessageIdPtr, int> idRefs;
-    QHash<TranslatorMessageContentPtr, int> contentRefs;
-    for (int i = 0; i < m_messages.count();) {
+    QSet<TranslatorMessageIdPtr> idRefs;
+    QSet<TranslatorMessageContentPtr> contentRefs;
+    for (int i = 0; i < m_messages.size();) {
         const TranslatorMessage &msg = m_messages.at(i);
         TranslatorMessage *omsg;
         int oi;
-        QSet<int> *pDup;
+        DuplicateEntries *pDup;
         if (!msg.id().isEmpty()) {
-            QHash<TranslatorMessageIdPtr, int>::ConstIterator it =
-                    idRefs.constFind(TranslatorMessageIdPtr(msg));
+            const auto it = idRefs.constFind(TranslatorMessageIdPtr(this, i));
             if (it != idRefs.constEnd()) {
-                oi = *it;
+                oi = it->messageIndex;
                 omsg = &m_messages[oi];
                 pDup = &dups.byId;
                 goto gotDupe;
             }
         }
         {
-            QHash<TranslatorMessageContentPtr, int>::ConstIterator it =
-                    contentRefs.constFind(TranslatorMessageContentPtr(msg));
+            const auto it = contentRefs.constFind(TranslatorMessageContentPtr(this, i));
             if (it != contentRefs.constEnd()) {
-                oi = *it;
+                oi = it->messageIndex;
                 omsg = &m_messages[oi];
                 if (msg.id().isEmpty() || omsg->id().isEmpty()) {
                     if (!msg.id().isEmpty() && omsg->id().isEmpty()) {
                         omsg->setId(msg.id());
-                        idRefs[TranslatorMessageIdPtr(*omsg)] = oi;
+                        idRefs.insert(TranslatorMessageIdPtr(this, oi));
                     }
                     pDup = &dups.byContents;
                     goto gotDupe;
@@ -604,12 +593,12 @@ Translator::Duplicates Translator::resolveDuplicates()
             }
         }
         if (!msg.id().isEmpty())
-            idRefs[TranslatorMessageIdPtr(msg)] = i;
-        contentRefs[TranslatorMessageContentPtr(msg)] = i;
+            idRefs.insert(TranslatorMessageIdPtr(this, i));
+        contentRefs.insert(TranslatorMessageContentPtr(this, i));
         ++i;
         continue;
       gotDupe:
-        pDup->insert(oi);
+        (*pDup)[oi].append(msg.tsLineNumber());
         if (!omsg->isTranslated() && msg.isTranslated())
             omsg->setTranslations(msg.translations());
         m_indexOk = false;
@@ -627,28 +616,43 @@ void Translator::reportDuplicates(const Duplicates &dupes,
             std::cerr << "'\n(try -verbose for more info).\n";
         } else {
             std::cerr << "':\n";
-            foreach (int i, dupes.byId)
-                std::cerr << "\n* ID: " << qPrintable(message(i).id()) << std::endl;
-            foreach (int j, dupes.byContents) {
-                const TranslatorMessage &msg = message(j);
+            for (auto it = dupes.byId.begin(); it != dupes.byId.end(); ++it) {
+                const TranslatorMessage &msg = message(it.key());
+                std::cerr << "\n* ID: " << qPrintable(msg.id()) << std::endl;
+                reportDuplicatesLines(msg, it.value());
+            }
+            for (auto it = dupes.byContents.begin(); it != dupes.byContents.end(); ++it) {
+                const TranslatorMessage &msg = message(it.key());
                 std::cerr << "\n* Context: " << qPrintable(msg.context())
                           << "\n* Source: " << qPrintable(msg.sourceText()) << std::endl;
                 if (!msg.comment().isEmpty())
                     std::cerr << "* Comment: " << qPrintable(msg.comment()) << std::endl;
+                reportDuplicatesLines(msg, it.value());
             }
             std::cerr << std::endl;
         }
     }
 }
 
+void Translator::reportDuplicatesLines(const TranslatorMessage &msg,
+                                       const DuplicateEntries::value_type &dups) const
+{
+    if (msg.tsLineNumber() >= 0) {
+        std::cerr << "* Line in .ts file: " << msg.tsLineNumber() << std::endl;
+        for (int tsLineNumber : dups) {
+            if (tsLineNumber >= 0)
+                std::cerr << "* Duplicate at line: " << tsLineNumber << std::endl;
+        }
+    }
+}
+
 // Used by lupdate to be able to search using absolute paths during merging
 void Translator::makeFileNamesAbsolute(const QDir &originalPath)
 {
-    for (TMM::iterator it = m_messages.begin(); it != m_messages.end(); ++it) {
-        TranslatorMessage &msg = *it;
-        TranslatorMessage::References refs = msg.allReferences();
+    for (auto &msg : m_messages) {
+        const TranslatorMessage::References refs = msg.allReferences();
         msg.setReferences(TranslatorMessage::References());
-        foreach (const TranslatorMessage::Reference &ref, refs) {
+        for (const TranslatorMessage::Reference &ref : refs) {
             QString fileName = ref.fileName();
             QFileInfo fi (fileName);
             if (fi.isRelative())
@@ -658,7 +662,7 @@ void Translator::makeFileNamesAbsolute(const QDir &originalPath)
     }
 }
 
-QList<TranslatorMessage> Translator::messages() const
+const QList<TranslatorMessage> &Translator::messages() const
 {
     return m_messages;
 }
@@ -670,11 +674,11 @@ QStringList Translator::normalizedTranslations(const TranslatorMessage &msg, int
 
     // make sure that the stringlist always have the size of the
     // language's current numerus, or 1 if its not plural
-    if (translations.count() > numTranslations) {
-        for (int i = translations.count(); i > numTranslations; --i)
+    if (translations.size() > numTranslations) {
+        for (int i = translations.size(); i > numTranslations; --i)
             translations.removeLast();
-    } else if (translations.count() < numTranslations) {
-        for (int i = translations.count(); i < numTranslations; ++i)
+    } else if (translations.size() < numTranslations) {
+        for (int i = translations.size(); i < numTranslations; ++i)
             translations.append(QString());
     }
     return translations;
@@ -684,22 +688,22 @@ void Translator::normalizeTranslations(ConversionData &cd)
 {
     bool truncated = false;
     QLocale::Language l;
-    QLocale::Country c;
-    languageAndCountry(languageCode(), &l, &c);
+    QLocale::Territory c;
+    languageAndTerritory(languageCode(), &l, &c);
     int numPlurals = 1;
     if (l != QLocale::C) {
         QStringList forms;
         if (getNumerusInfo(l, c, 0, &forms, 0))
-            numPlurals = forms.count(); // includes singular
+            numPlurals = forms.size(); // includes singular
     }
-    for (int i = 0; i < m_messages.count(); ++i) {
+    for (int i = 0; i < m_messages.size(); ++i) {
         const TranslatorMessage &msg = m_messages.at(i);
         QStringList tlns = msg.translations();
         int ccnt = msg.isPlural() ? numPlurals : 1;
-        if (tlns.count() != ccnt) {
-            while (tlns.count() < ccnt)
+        if (tlns.size() != ccnt) {
+            while (tlns.size() < ccnt)
                 tlns.append(QString());
-            while (tlns.count() > ccnt) {
+            while (tlns.size() > ccnt) {
                 tlns.removeLast();
                 truncated = true;
             }
@@ -716,13 +720,13 @@ void Translator::normalizeTranslations(ConversionData &cd)
 QString Translator::guessLanguageCodeFromFileName(const QString &filename)
 {
     QString str = filename;
-    foreach (const FileFormat &format, registeredFileFormats()) {
+    for (const FileFormat &format : std::as_const(registeredFileFormats())) {
         if (str.endsWith(format.extension)) {
             str = str.left(str.size() - format.extension.size() - 1);
             break;
         }
     }
-    static QRegExp re(QLatin1String("[\\._]"));
+    static QRegularExpression re(QLatin1String("[\\._]"));
     while (true) {
         QLocale locale(str);
         //qDebug() << "LANGUAGE FROM " << str << "LANG: " << locale.language();
@@ -739,6 +743,27 @@ QString Translator::guessLanguageCodeFromFileName(const QString &filename)
     return QString();
 }
 
+void Translator::appendDependencies(const QStringList &dependencies)
+{
+    QStringList mergeDeps;
+    for (const QString &dep : dependencies) {
+        if (const auto it = std::find(m_dependencies.cbegin(), m_dependencies.cend(), dep);
+            it == m_dependencies.cend()) {
+            mergeDeps.append(dep);
+        }
+    }
+    m_dependencies.append(mergeDeps);
+}
+
+void Translator::satisfyDependency(const QString &file, const QString &format)
+{
+    const auto dep = getDependencyName(file, format);
+    if (const auto it = std::find(m_dependencies.cbegin(), m_dependencies.cend(), dep);
+        it != m_dependencies.cend()) {
+        m_dependencies.erase(it);
+    }
+}
+
 bool Translator::hasExtra(const QString &key) const
 {
     return m_extra.contains(key);
